"""This module contains the AnalysisInputResponse domain model."""

from mythx_models.request.analysis_submission import AnalysisSubmissionRequest


class AnalysisInputResponse(AnalysisSubmissionRequest):
    pass
