"""This module contains the GroupOperationResponse domain model."""

from mythx_models.response.group_status import GroupStatusResponse


class GroupOperationResponse(GroupStatusResponse):
    """The API response domain model for the status of a group after the
    operation."""

    pass
